;================================================================
;
;  Heater service
;  version 1.0
;  
;  oscillator 4MHz 
;
;  Microchip MPLAB IDE compiler   /www.microchip.com/
;
;================================================================

#include <P16F676.inc>

BG0_OFF EQU 0x2FFF
BG1_OFF EQU 0x1FFF

 __CONFIG _XT_OSC & _CPD_OFF & _CP_OFF & _BODEN_OFF & _MCLRE_OFF & _PWRTE_OFF & _WDT_OFF

rxlength EQU 0x0A

 cblock 0x20
  rxbuf: rxlength
  rxchar, rxbit, txchar, txbit
  TMPR, TMPD,TMPE,CTMP0,DTMP1,DTMP2,DTMP3,COMTOUT
  flags, flags1
  sec, min, hour, dweek
  HiTmr,LoTmr
  fTemp, Temp, T1, T2, T3 
  EIdx, ETemp, EDWeek, EHour, EMin, ESensor, EPump
  RIdx, RTemp, RDWeek, RHour, RMin, RSensor, RPump
  EAddr, FIdx, TTemp, TIdx
  beepcnt
  i2cbit, i2cerr, i2cdev
 endc

i2cdel   EQU TMPE
i2cxbyte EQU DTMP1
i2crbyte EQU DTMP2
i2cbdev  EQU .144

TMPW     EQU 0x5D         ; push/pop W on interrupt
TMPS     EQU 0x5E         ; push/pop STATUS
TMPFSR   EQU 0x5F         ; push/pop FSR


#define sdaPort PORTA
#define sclPort PORTA
#define sdaTRIS TRISA


#define RXPort PORTA
#define TXPort PORTC
#define HeatPort PORTC
#define PumpPort0 PORTC
#define PumpPort1 PORTC
#define PumpPort2 PORTC
#define BeepPort PORTA
#define PortOnBatt PORTC

PortDirA EQU  b'00111000'
; PORTA
sdaPin   EQU 0x0 
sclPin   EQU 0x1 
Beep     EQU 0x2 
RXLine   EQU 0x3 
OSC1     EQU 0x4 
OSC2     EQU 0x5

PortDirC EQU b'00001000'
; PORTC
Pump2    EQU 0x0 
Pump1    EQU 0x1 
Pump0    EQU 0x2 
OnBatt   EQU 0x3 
HeatOn   EQU 0x4 
TXLine   EQU 0x5 

ctout    EQU .4    ; RX timeout

delbit1  EQU .13  
delbit2  EQU .19            

; flags
fOnBatt        EQU 0x00
fds18err       EQU 0x01
fUpdateTemp    EQU 0x02
fSummerTime    EQU 0x03
fHeatOFF       EQU 0x04
fHeatON        EQU 0x05
fFoundRule     EQU 0x06
fDebugPrint    EQU 0x07

; flags1
fTempChg       EQU 0x00 
fHeaterChg     EQU 0x01 
fminusTemp     EQU 0x02 
fFlag3         EQU 0x03
fFlag4         EQU 0x04
fFlag5         EQU 0x05
fFlag6         EQU 0x06
fFlag7         EQU 0x07

HiTmrEpromAddr EQU .0
LoTmrEpromAddr EQU .1

cmp macro var1,const1
     movfw var1
     sublw const1
 endm

#define disable_bank_warn errorlevel -302
#define enable_bank_warn  errorlevel +302

     ORG     0x2100       ; eeprom data 
     de 0x0B   ; Hi crystal timer (3036)  0x0B
     de 0xDC   ; Lo crystal timer         0xDC
     de .0   ; 
     de .0   ;
     de .0   ;
     de .0   ;
     de .0   ;
     de .0   ; 
     
     ORG 0x2108 ; eeprom rules 120 bytes
     ; idx, temp, weekday, hour, minute, senzor, pump
     de 0x01,0x0A,0x00,0x00,0x00,0x00,0x00,0
     de 0x02,0x0A,0x00,0x00,0x00,0x00,0x00,0
     de 0x03,0x0A,0x00,0x00,0x00,0x00,0x00,0  
     de 0x04,0x0A,0x00,0x00,0x00,0x00,0x00,0
     de 0x05,0x0A,0x00,0x00,0x00,0x00,0x00,0
     de 0x06,0x0A,0x00,0x00,0x00,0x00,0x00,0
     de 0x07,0x0A,0x00,0x00,0x00,0x00,0x00,0
     de 0x08,0x0A,0x00,0x00,0x00,0x00,0x00,0
     de 0x09,0x0A,0x00,0x00,0x00,0x00,0x00,0
     de 0x0A,0x0A,0x00,0x00,0x00,0x00,0x00,0
     de 0x0B,0x0A,0x00,0x00,0x00,0x00,0x00,0
     de 0x0C,0x0A,0x00,0x00,0x00,0x00,0x00,0
     de 0x0D,0x0A,0x00,0x00,0x00,0x00,0x00,0
     de 0x0E,0x0A,0x00,0x00,0x00,0x00,0x00,0
     de 0x0F,0x0A,0x00,0x00,0x00,0x00,0x00,0

;================================================================
;     ORG 0x3FF            ; OSCCAL
;     retlw 0x34           ; device depended value
;================================================================

     ORG     0x0
     goto    Start
 
     ORG     0x04        
Interrupt                
     movwf TMPW
     swapf STATUS,W
     bcf   STATUS,RP0    
     movwf TMPS
     movfw FSR
     movwf TMPFSR
     btfsc INTCON,RAIF   
     goto  DOPCInt
     btfsc INTCON,INTF   
     goto  DoINT0
     btfsc INTCON,T0IF   
     goto  DoTimer0
     btfsc PIR1,TMR1IF   
     goto  DoTimer1

IEEnd                    
     movfw TMPFSR
     movwf FSR
     swapf TMPS,W
     movwf STATUS
     swapf TMPW,F
     swapf TMPW,W
     retfie


DoTimer0
     bcf   INTCON,T0IF
     goto  IEEnd

UpdateTime
     incf  sec,f
     movlw .120
     subwf sec,w
     btfss STATUS,Z
     return
     clrf  sec
     bsf   flags,fUpdateTemp 
     incf  min,f
     movlw .60
     subwf min,w
     btfss STATUS,Z
     return
     clrf  min
     incf  hour,f
     movlw .24
     subwf hour,w
     btfss STATUS,Z
     return
     clrf  hour
     incf  dweek,f
     movfw dweek
     sublw .7
     btfsc STATUS,Z
     clrf  dweek
     return

DoTimer1
     call  SetTimer1
     call  UpdateTime    
     movfw COMTOUT
     btfsc STATUS,Z
     goto  dtmr1
     decf  COMTOUT,F 
     goto  DoTimer1Ex
dtmr1
     clrf  rxbuf
DoTimer1Ex
     bcf   PIR1,TMR1IF
     goto  IEEnd

DOPCInt
     call USARTReceive
     bcf  INTCON,RAIF
     goto IEEnd

DoINT0                    
     bcf   INTCON,INTF
     goto  IEEnd



DelayBit1_0           
     movlw delbit1    
     movwf TMPD       
N104 decfsz TMPD,F    
     goto  N104       
     return           

DelayBit1_5           
     movlw delbit2    
     movwf TMPD       
N156 decfsz TMPD,F    
     goto  N156       
     return           


SendChar
     movwf txchar
     movlw .8
     movwf txbit
     bcf  INTCON,GIE  
     bcf  TXPort,TXLine
     call DelayBit1_0 
SendChar0
     rrf  txchar,F
     btfsc STATUS,C
     goto SendChar1
     bcf  TXPort,TXLine
     goto SendChar2
SendChar1
     bsf TXPort,TXLine
SendChar2
     call DelayBit1_0
     decfsz txbit,F
     goto SendChar0
     bsf  TXPort,TXLine
     bsf  INTCON,GIE  
     call DelayBit1_0 
     call DelayBit1_0 
     return


InitTimer1 
     clrf  sec
     clrf  min
     clrf  hour
     clrf  dweek
     bcf   STATUS,RP0
     movlw b'00110100'
     movwf T1CON
     bsf   STATUS,RP0 
     bsf   PIE1,TMR1IE
     bcf   STATUS,RP0 
     bcf   PIR1,TMR1IF
     return

SetTimer1
     bcf   STATUS,RP0 
     bcf   T1CON,TMR1ON
     movfw HiTmr
     movwf TMR1H       
     movfw LoTmr
     movwf TMR1L
     movlw b'00110100'
     movwf T1CON
     bsf   T1CON,TMR1ON 
     return




Start
     bsf   STATUS,RP0
     call  0x3FF
     movwf OSCCAL
     bcf   STATUS,RP0

     movlw b'00000111'
     movwf CMCON     
     bsf   STATUS,RP0
     clrf  IOCA
     bsf   IOCA,RXLine 
     bcf   OPTION_REG,NOT_RAPU

     clrf  ANSEL
     movlw PortDirA
     movwf TRISA    
     movlw PortDirC
     movwf TRISC
     bcf   STATUS,RP0   

     clrf  PORTA
     clrf  PORTC
     bcf   HeatPort,HeatOn 
     bsf   TXPort,TXLine
     bsf   RXPort,RXLine
     call  i2cinit

     clrf  flags
     clrf  flags1
     clrf  rxbuf
     clrf  rxbit
     clrf  COMTOUT
     call  ClearRVar

     movlw HiTmrEpromAddr
     call  ReadEprom
     movwf HiTmr
     movlw LoTmrEpromAddr
     call  ReadEprom
     movwf LoTmr
     call  InitTimer1

     movlw b'11001000' 
     movwf INTCON         
     bcf   INTCON,RAIF 
     bsf   T1CON,TMR1ON 
     call  Delay200ms   
     call  Delay200ms   
     call  GetTemperatures
     bsf   flags, fOnBatt

Loop
 
     clrwdt
     call  CheckUSART
 
     btfsc flags,fOnBatt
     call  BeepAlert

     btfss PortOnBatt,OnBatt
     goto  RunOnBatt
     
     btfss flags,fUpdateTemp
     goto  Loop
     bcf   flags,fUpdateTemp
     call  GetTemperatures
     btfsc flags,fSummerTime
     goto  LoopSummer

     call  CheckPrograms
     movfw RDWeek      
     andlw .128
     btfsc STATUS,Z    
     goto  LoopSummer  

     movlw (1 << fHeatON) + (1 << fHeatOFF)
     andwf flags,w
     btfss STATUS,Z
     goto  LoopOnOff   

     call  CheckSensorTemp
     movfw fTemp          
     btfss STATUS,Z
     call  PumpSetOnOff
     movfw fTemp
     btfsc STATUS,Z
     call  CheckPump   

     goto  Loop
LoopOnOff
     btfss flags,fHeatON
     goto  LoopOnOff1
     bsf   HeatPort, HeatOn
     call  PumpSetOn
LoopOnOff1
     btfss flags,fHeatOFF
     goto  LoopOnOff2
     bcf   HeatPort, HeatOn 
     call  CheckPump    
LoopOnOff2
     goto  Loop

LoopSummer              
     call  CheckPump
     goto  Loop

RunOnBatt
     bsf   flags, fOnBatt
     bcf   HeatPort, HeatOn 
     call  PumpSetAllOff    
     goto  Loop


BeepAlert           
     call  Delay50ms
     incf  beepcnt,f
     movfw beepcnt
     btfss STATUS,Z
     return
     movlw .156             
     movwf beepcnt
     movlw .100
     movwf TMPE
BeepAlert1                  
     bsf   BeepPort, Beep
     call  Delay1ms
     bcf   BeepPort, Beep
     call  Delay1ms
     decfsz TMPE,f
     goto  BeepAlert1
     return

CheckTemp  
     movwf Temp
     addlw .1
     btfsc STATUS,Z
     goto  CheckTemp1
     btfsc Temp,7    
     goto  CheckTemp0
     movfw RTemp
     subwf Temp,w
     btfsc STATUS,C
     goto  CheckTemp1
CheckTemp0
     bsf   STATUS,C
     return
CheckTemp1
     bcf   STATUS,C
     return

CheckSensorTemp

     clrf  fTemp
     movfw T1
     call  CheckTemp
     btfsc STATUS,C
     bsf   fTemp,0
CheckSensorTemp1
     movfw T2
     call  CheckTemp
     btfsc STATUS,C
     bsf   fTemp,1
CheckSensorTemp2
     movfw T3
     call  CheckTemp
     btfsc STATUS,C
     bsf   fTemp,2
CheckSensorTemp3
     return

CheckPumpOn
     movfw hour
     sublw .10 
     btfss STATUS,Z
     return
     movfw min
     sublw .4  
     btfsc STATUS,C
     call  PumpSetAllOn
     return

CheckPump
     bcf   HeatPort, HeatOn 
     movfw hour
     sublw .10              
     btfss STATUS,Z
     goto  CheckPump1           
     movfw min
     sublw .4               
     btfss STATUS,C
     goto  CheckPump1
     call  PumpSetAllOn
     return
CheckPump1
     call PumpSetAllOff
     return

IsPumpEnable
     movwf  Temp
     andwf  RPump,w
     btfsc  STATUS,Z      
     return               
     movfw  Temp        
     andwf  RSensor,w
     btfsc  STATUS,Z      
     goto   IsPumpEnable0 
     movfw  Temp          
     andwf  fTemp,w
     return               
IsPumpEnable0       
     bcf    STATUS,Z      
     return

PumpSetOnOff
      bsf   HeatPort, HeatOn 
     movlw .1
     call  IsPumpEnable
     btfss STATUS,Z
     bsf   PumpPort0, Pump0
     btfsc STATUS,Z
     bcf   PumpPort0, Pump0
     movlw .2
     call  IsPumpEnable
     btfss STATUS,Z
     bsf   PumpPort1, Pump1
     btfsc STATUS,Z
     bcf   PumpPort1, Pump1
     movlw .4
     call  IsPumpEnable
     btfss STATUS,Z
     bsf   PumpPort2, Pump2
     btfsc STATUS,Z
     bcf   PumpPort2, Pump2

     call  CheckPumpOn
     return

PumpSetOn
     btfsc RPump,0
     bsf   PumpPort0, Pump0
     btfss RPump,0
     bcf   PumpPort0, Pump0
     btfsc RPump,1
     bsf   PumpPort1, Pump1
     btfss RPump,1
     bcf   PumpPort1, Pump1
     btfsc RPump,2
     bsf   PumpPort2, Pump2
     btfss RPump,2
     bcf   PumpPort2, Pump2
     return

PumpSetAllOff
     bcf   PumpPort0, Pump0     
     bcf   PumpPort1, Pump1
     bcf   PumpPort2, Pump2
     return

PumpSetAllOn
     bsf   PumpPort0, Pump0     
     bsf   PumpPort1, Pump1
     bsf   PumpPort2, Pump2
     return


ClearRVar
     clrf  RIdx
     clrf  RTemp
     clrf  RDWeek
     clrf  RHour
     clrf  RMin
     clrf  RSensor
     clrf  RPump
     return

CheckPrograms
     call ClearRVar
     bcf  flags,fFoundRule
     bcf  flags,fHeatON
     bcf  flags,fHeatOFF
     clrf FIdx
CheckPrograms1
     incf  FIdx,f
     movlw .8
     movwf EAddr
     movlw .15
     movwf TMPR   
CheckPrograms2

     call  ReadRule 
     movfw EIdx
     subwf FIdx,w
     btfss STATUS,Z 
     goto  CheckPrograms3
     movfw EDWeek
     andlw .128
     btfsc STATUS,Z  
     goto  CheckPrograms3

     movlw HIGH ConvertBits
     movwf PCLATH
     movfw dweek
     call  ConvertBits
     andwf EDWeek,w    
     btfsc STATUS,Z  
     goto  CheckPrograms3

     movfw EHour
     andlw b'00111111'  
     subwf hour,w
     btfsc STATUS,Z
     goto  CheckRuleMin
     btfss STATUS,C   
     goto  CheckPrograms3
     goto  CheckRule
CheckRuleMin
     movfw min
     subwf EMin,w
     btfss STATUS,C
     goto  CheckRule
     btfss STATUS,Z
     goto  CheckPrograms3
CheckRule
     bsf  flags,fFoundRule
     movfw EIdx
     movwf RIdx
     movfw ETemp
     movwf RTemp
     movfw EDWeek
     movwf RDWeek
     movfw EHour
     movwf RHour
     movfw EMin
     movwf RMin
     movfw ESensor
     movwf RSensor
     movfw EPump
     movwf RPump
     
     btfss flags,fDebugPrint
     goto  CheckPrograms3
     call  Delay200ms
     call  SendRule     
CheckPrograms3
     decfsz TMPR,f
     goto  CheckPrograms2
CheckPrograms4
     movlw .15        
     subwf FIdx,w
     btfss STATUS,Z
     goto  CheckPrograms1 

     btfss flags,fFoundRule
     return
     movfw RHour
     andlw b'10000000'
     btfss STATUS,Z
     bsf   flags,fHeatON
     movfw RHour
     andlw b'01000000'
     btfss STATUS,Z
     bsf   flags,fHeatOFF
     return

SendRule
     movfw RIdx
     call  SendChar
     movfw RTemp
     call  SendChar
     movfw RDWeek
     call  SendChar
     movfw RHour
     call  SendChar
     movfw RMin
     call  SendChar
     movfw RSensor
     call  SendChar
     movfw RPump
     call  SendChar
     return

ReadEAddr
     movfw EAddr
     incf  EAddr,f
     call  ReadEprom
     return
ReadRule
     call  ReadEAddr
     movwf EIdx
     call  ReadEAddr
     movwf ETemp
     call  ReadEAddr
     movwf EDWeek
     call  ReadEAddr
     movwf EHour
     call  ReadEAddr
     movwf EMin
     call  ReadEAddr
     movwf ESensor
     call  ReadEAddr
     movwf EPump
     incf  EAddr,f
     return

;==========================================================
; LM75
;==========================================================

SetI2CDev      
     andlw b'00000111'
     movwf i2cdev
     bcf   STATUS,C
     rlf   i2cdev,f     
     movlw i2cbdev
     addwf i2cdev,f
     return

TempError
     call i2cstop
     movlw .1
     movwf i2cerr
     movlw .255
     movwf Temp
     return

ReadTemperature    
     call  SetI2CDev
     clrf  i2cerr
     call  i2cstart
     movfw i2cdev  
     call  i2csendbyte
     btfss STATUS,C
     goto  TempError
     movlw .0   
     call  i2csendbyte
     btfss STATUS,C
     goto  TempError
     call  i2crestart
     movlw .1
     addwf i2cdev,w
     call  i2csendbyte
     btfss STATUS,C
     goto  TempError
     call  i2creceiveackbyte
     movfw i2crbyte
     movwf Temp
     call  i2creceivenoackbyte
     call  i2cstop

     btfss flags,fDebugPrint
     goto  PlusTemp
     movfw Temp              
     call SendChar
     movfw i2crbyte
     call SendChar
PlusTemp
     btfsc Temp,7  
     goto  MinusTemp
     call  TempShr5
     bcf   flags1,fminusTemp
     goto  CorrectTemp
MinusTemp
     bsf   flags1,fminusTemp
     comf  Temp,f
     movlw b'01111111'
     andwf Temp,f
     comf  i2crbyte,f 
     call  TempShr5
     incf  i2crbyte,f 
     movlw .8
     subwf i2crbyte,w
     btfss STATUS,Z
     goto  CorrectTemp
     incf  Temp,f
     clrf  i2crbyte
CorrectTemp
     bcf   STATUS,C
     rlf   Temp,f
     movlw .4
     subwf i2crbyte,w 
     btfsc STATUS,C
     bsf   Temp,0
     btfsc flags1,fminusTemp
     bsf   Temp,7     
     movfw Temp
     return

TempShr5
     bcf   STATUS,C     
     rrf   i2crbyte,f 
     bcf   STATUS,C
     rrf   i2crbyte,f
     bcf   STATUS,C
     rrf   i2crbyte,f
     bcf   STATUS,C
     rrf   i2crbyte,f
     bcf   STATUS,C
     rrf   i2crbyte,f
     return

;==========================================================

GetTemperatures
     movlw .0
     call  ReadTemperature
     movwf T1
GetTemperatures1
     movlw .1
     call  ReadTemperature
     movwf T2
GetTemperatures2
     movlw .2
     call  ReadTemperature
     movwf T3
GetTemperatures3
     return
     

USARTReceive
     btfsc RXPort,RXLine 
     return              
     movlw .8            
     movwf rxbit         
     clrf  rxchar        
     call  DelayBit1_5
CheckUSARTReceive0
     bcf   STATUS,C      
     btfsc RXPort,RXLine 
     bsf   STATUS,C
     rrf   rxchar,F      
     call  DelayBit1_0   
     decfsz rxbit,F      
     goto  CheckUSARTReceive0
     movlw ctout
     movwf COMTOUT
     movlw rxlength
     subwf rxbuf,W
     btfsc STATUS,C      
     return
     incf  rxbuf,F		 
     movlw rxbuf		 
     addwf rxbuf,W       
     movwf FSR           
     movfw rxchar        
     movwf INDF          
     return


CheckUSART
     movlw rxlength
     subwf rxbuf,W
     btfss STATUS,C    
     return
     bcf INTCON,RAIE   

cmd0    
     cmp rxbuf+1,.0    
     btfss STATUS,Z		
	 goto  cmd1
     goto  CheckUSARTEx
cmd1
     cmp rxbuf+1,.1
     btfss STATUS,Z		
	 goto  cmd2
     call  SendVersion
     goto  CheckUSARTEx
cmd2
     cmp rxbuf+1,.2
     btfss STATUS,Z		
	 goto  cmd3
     call  GetState
     goto  CheckUSARTEx
cmd3
     cmp rxbuf+1,.3
     btfss STATUS,Z		
	 goto  cmd4
     call  GetTemperatures
     goto  CheckUSARTEx
cmd4
     cmp rxbuf+1,.4
     btfss STATUS,Z		
	 goto  cmd5
     call  WriteEpromByte
     goto  CheckUSARTEx
cmd5
     cmp rxbuf+1,.5
     btfss STATUS,Z		
	 goto  cmd6
     call  ReadEpromByte
     goto  CheckUSARTEx
cmd6
     cmp rxbuf+1,.6
     btfss STATUS,Z		
	 goto  cmd7
     call  WriteEpromByte8
     goto  CheckUSARTEx
cmd7
     cmp rxbuf+1,.7
     btfss STATUS,Z		
	 goto  cmd8
     call  ReadEpromByte8
     goto  CheckUSARTEx
cmd8
     cmp rxbuf+1,.8
     btfss STATUS,Z		
	 goto  cmd9
     bsf   flags1,fTempChg
     movfw rxbuf+2
     movwf TTemp
     goto  CheckUSARTEx
cmd9
     cmp rxbuf+1,.9
     btfss STATUS,Z		
	 goto  cmd10
     movfw rxbuf+2
     btfsc STATUS,Z
     bcf   flags,fDebugPrint
     btfss STATUS,Z
     bsf   flags,fDebugPrint
     goto  CheckUSARTEx
cmd10
     cmp rxbuf+1,.10
     btfss STATUS,Z		
	 goto  cmd11
     bsf   flags1,fHeaterChg
     movfw rxbuf+2
     btfsc STATUS,Z
     bcf   HeatPort, HeatOn
     btfss STATUS,Z
     bsf   HeatPort, HeatOn
     goto  CheckUSARTEx
cmd11
     cmp rxbuf+1,.11
     btfss STATUS,Z		
	 goto  cmd12
     movfw rxbuf+2
;     movwf cent
     movfw rxbuf+3
;     movwf year
     movfw rxbuf+4
;     movwf month
     movfw rxbuf+5
;     movwf day
     movfw rxbuf+6
     movwf dweek
     movfw rxbuf+7
     movwf hour
     movfw rxbuf+8
     movwf min
     movfw rxbuf+9
     movwf sec
     bcf   flags, fOnBatt   
     goto  CheckUSARTEx
cmd12
     cmp rxbuf+1,.12
     btfss STATUS,Z		
	 goto  cmd13
;     movfw cent
     movlw .20
     call  SendChar
     movlw .14
;     movfw year
     call  SendChar
     movlw .1
;     movfw month
     call  SendChar
     movlw .1
;     movfw day
     call  SendChar
     movfw dweek
     call  SendChar

     movfw hour
     call  SendChar
     movfw min
     call  SendChar
     movfw sec
     call  SendChar
     bcf   flags, fOnBatt 
     goto  CheckUSARTEx
cmd13
     cmp rxbuf+1,.13
     btfss STATUS,Z		
	 goto  cmd14
     movlw HiTmrEpromAddr
     call  SetEpromAddr
     movfw rxbuf+2
     movwf HiTmr
     call  WriteEprom
     movlw LoTmrEpromAddr
     call  SetEpromAddr
     movfw rxbuf+3
     movwf LoTmr
     call  WriteEprom
     goto  CheckUSARTEx
cmd14
     cmp rxbuf+1,.14
     btfss STATUS,Z		
	 goto  cmd15
     call  SendRule
     goto  CheckUSARTEx
cmd15
     cmp rxbuf+1,.15
     btfss STATUS,Z		
	 goto  cmd16
     bsf   flags,fUpdateTemp
     goto  CheckUSARTEx
cmd16
     cmp rxbuf+1,.16
     btfss STATUS,Z		
	 goto  cmd17
     goto  CheckUSARTEx
cmd17
   
CheckUSARTEx
     call  SendCRLF
     clrf  rxbuf
     bsf   INTCON,RAIE 
     return


GetState
     movfw PORTC
     call  SendChar
     movfw PORTA
     call  SendChar
     movfw flags
     call  SendChar
     movfw flags1
     call  SendChar
     movfw HiTmr
     call  SendChar
     movfw LoTmr
     call  SendChar
     movfw T1
     call  SendChar
     movfw T2
     call  SendChar
     movfw T3
     call  SendChar
     return

SendVersion
     clrf  CTMP0
SendVersion1
     movlw HIGH version
     movwf PCLATH
     movfw CTMP0
     call  version
     andlw 0xFF
     btfsc STATUS,Z
     return
     call  SendChar        
     incf  CTMP0,F
     goto  SendVersion1


SendCRLF
     movlw .13
     call  SendChar
     movlw .10
     call  SendChar
     return


Delay_
     movwf DTMP1
N0   movlw .100  
     movwf DTMP2
N1   movlw .166  
     movwf DTMP3
N2   decfsz DTMP3,F                
     goto  N2
     decfsz DTMP2,F
     goto  N1
     decfsz DTMP1,F                
     goto  N0
     return

Delay1ms
Nn0  movlw .65
     movwf DTMP2
Nn1  movlw .2
     movwf DTMP3
Nn2  decfsz DTMP3,F                
     goto  Nn2
     decfsz DTMP2,F
     goto  Nn1
     return

Delay50ms
     movlw .1
     call  Delay_
     return

Delay100ms
     movlw .2
     call  Delay_
     return

Delay200ms
     movlw .4
     call  Delay_
     return


WriteEpromByte
     movfw rxbuf+2
     call  SetEpromAddr
     movfw rxbuf+3
     call  WriteEprom
     return

ReadEpromByte
     movfw rxbuf+2
     call  ReadEprom
     call  SendChar
     return

WriteEpromByte8
     movlw rxbuf+3   
     movwf FSR       
     movlw .8
     movwf TMPE
web8
     movfw rxbuf+2   
     call  SetEpromAddr
     movfw INDF	     
     call  WriteEprom
     incf  FSR,F
     incf  rxbuf+2,F
     decfsz TMPE,F
     goto web8
     return

ReadEpromByte8
     movlw .8
     movwf TMPE
reb8
     movfw rxbuf+2   
     call  ReadEprom
     call  SendChar
     incf  rxbuf+2,F
     decfsz TMPE,F
     goto reb8
     return


 disable_bank_warn


ReadEprom                 
     bsf   STATUS, RP0    
     movwf EEADR          
     bsf   EECON1, RD     
     movfw EEDATA         
     bcf   STATUS, RP0    
     return

SetEpromAddr              
     bsf   STATUS, RP0    
     movwf EEADR          
     bcf   STATUS, RP0    
     return

WriteEprom                
     bsf   STATUS, RP0    
     movwf EEDATA         
     bsf   EECON1, WREN   
     bcf   INTCON, GIE    
     movlw 0x55           
     movwf EECON2         
     movlw 0xAA           
     movwf EECON2         
     bsf   EECON1,WR      
                          
     bsf   INTCON, GIE    
     bcf   EECON1, WREN   
     btfsc EECON1,WR      
     goto  $-1
     bcf   STATUS, RP0    
     bcf   PIR1,EEIF      
     return


SetOutSDA
     bsf    STATUS,RP0    
     bcf    sdaTRIS,sdaPin
     bcf    STATUS,RP0    
     return

SetInSDA
     bsf    STATUS,RP0    
     bsf    sdaTRIS,sdaPin
     bcf    STATUS,RP0    
     return

i2cdelay          
     movlw .20
     movwf i2cdel
i2cdelay0
     decfsz i2cdel,F                
     goto  i2cdelay0
     return

i2cdelay2         
     movlw .10
     movwf i2cdel
i2cdelay1
     decfsz i2cdel,F                
     goto  i2cdelay1
     return

i2cinit           
     bsf   sdaPort,sdaPin
     bsf   sclPort,sclPin
     movlw i2cbdev
     movwf i2cdev
     clrf  i2cerr
     return

i2cstart          
     call  SetOutSDA
     bsf   sclPort,sclPin
     call  i2cdelay2 
     bsf   sdaPort,sdaPin  
     call  i2cdelay
     bcf   sdaPort,sdaPin       
     call  i2cdelay
     return

i2crestart        
     call  SetOutSDA  
     bcf   sclPort,sclPin
     call  i2cdelay
     bcf   sdaPort,sdaPin
     call  i2cdelay
     bsf   sdaPort,sdaPin
     call  i2cdelay
     bsf   sclPort,sclPin
     call  i2cdelay
     bcf   sdaPort,sdaPin       
     call  i2cdelay
     return

i2cstop             
     bcf   sclPort,sclPin
     call  i2cdelay   
     bcf   sdaPort,sdaPin
     call  i2cdelay
     bsf   sclPort,sclPin
     call  i2cdelay
     bsf   sdaPort,sdaPin
     call  i2cdelay
     return

i2csendack
     bcf   sclPort,sclPin
     call  i2cdelay2
     bcf   sdaPort,sdaPin    
     call  i2cdelay2
     bsf   sclPort,sclPin
     call  i2cdelay
     return

i2cgetack
     bcf   sclPort,sclPin
     call  i2cdelay2
     bsf   sdaPort,sdaPin
     call  i2cdelay2
     bsf   sclPort,sclPin
     call  SetInSDA
     call  i2cdelay     
     bcf   STATUS,C
     btfss sdaPort,sdaPin
     bsf   STATUS,C
     call  SetOutSDA
     return

i2csendbyte   
     movwf i2cxbyte
     movlw .8
     movwf i2cbit
i2csendbyte1
     bcf   sclPort,sclPin
     call  i2cdelay2
     rlf   i2cxbyte,f
     bcf   sdaPort,sdaPin  
     btfsc STATUS,C
     bsf   sdaPort,sdaPin
     call  i2cdelay2
     bsf   sclPort,sclPin
     call  i2cdelay
     decfsz i2cbit,f
     goto  i2csendbyte1
     call  i2cgetack
     return

i2creceiveackbyte
     bsf   sdaPort,sdaPin
     movlw .8
     movwf i2cbit
     clrf  i2crbyte
     call  SetInSDA
i2creceivebyte1
     bcf   sclPort,sclPin
     call  i2cdelay
     bsf   sclPort,sclPin
     call  i2cdelay
     bcf   STATUS,C   
     btfsc sdaPort,sdaPin
     bsf   STATUS,C
     rlf   i2crbyte,f
     decfsz i2cbit,f
     goto  i2creceivebyte1
     call  SetOutSDA
     call  i2csendack
     return

i2creceivenoackbyte
     bsf   sdaPort,sdaPin
     movlw .8
     movwf i2cbit
     clrf  i2crbyte
     call  SetInSDA
i2creceivebyte11
     bcf   sclPort,sclPin
     call  i2cdelay
     bsf   sclPort,sclPin
     call  i2cdelay
     bcf   STATUS,C   
     btfsc sdaPort,sdaPin
     bsf   STATUS,C
     rlf   i2crbyte,f
     decfsz i2cbit,f
     goto  i2creceivebyte11

     bcf   sclPort,sclPin
     call  i2cdelay
     bsf   sclPort,sclPin
     call  i2cdelay
     call  SetOutSDA
     return



     ORG 0x3D0
ConvertBits
     addwf PCL,F
     dt .1, .2, .4, .8, .16, .32, .64, .128

     ORG 0x3E0
version   
     addwf PCL,F
     dt "Heater service v1.0",0


 END

